

#    experiment run control for R3B - adapted from AIDA

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0

global Options

global NumASICs MaxChans
set NumASICs 16
set MaxChans 128


variable SpecBase [file join $env(MIDASBASE) SpecData R3B]
variable SpecPath

proc Dump_Raw_Data {} {return "[Dump_Raw_R3B_Data]\n"}

proc Set_Options {} {
   global Options

   LoadOptions

   if {![info exists Options(R3B_GroupBase)]} {set Options(R3B_GroupBase) 1}
   if {![info exists Options(Include.R3B)]}   {set Options(Include.R3B) 1}

   if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) 2048}
   if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) 2048}
   if {![info exists Options(Stat.shift)]}    {set Options(Stat.shift) 0}
   if {![info exists Options(Stat.offset)]}   {set Options(Stat.offset) 128}

   if {![info exists Options(R3B.channels)]} {set Options(R3B.channels) 4096}
   if {![info exists Options(R3B.shift)]}    {set Options(R3B.shift) 0}
   if {![info exists Options(R3B.offset)]}   {set Options(R3B.offset) 0}
    
   if {![info exists Options(R3B_Hist_E_Enable)]} {set Options(R3B_Hist_E_Enable) 1}

   SaveOptions

}

proc Dump_Raw_R3B_Data {} {

   set info "data $::DataAcquisition::ACQSERVER\n"

   for {set offset 0} {$offset <= 500} {incr offset 100} {

     DataAcqCommand 19 "Getting Raw Data" "19 $offset" 2
     set data [ReadNetVar NetVar.EXEC.Response]

     set c 0

     foreach word $data {
         if {$word == 0xaa5555aa} then {
             append info "\n$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "\n$word - end of event\n"
             continue
         }
         append info "$word "
         incr c

         if {$c == 10} then {append info "\n"; set c 0}
     }
   }

   return "$info"
}

proc create_R3B_spectra {} {
   global Options
   global NumASICs MaxChans

   set Me $::DataAcquisition::ACQSERVER

   for {set Asic 0} {$Asic < $NumASICs} {incr Asic} {

      puts "Creating spectra for ASIC $Asic"

      for {set Ch 0} {$Ch < $MaxChans} {incr Ch} {

        if {$Options(R3B_Hist_E_Enable) == 1} {

        set z [catch {CreateSpectrum1D E.$Asic.$Ch $Options(R3B.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Error attempting to create spectrum $E.$Asic.$Ch"
            error $m
         }

         set z [catch {SpectrumPutTitle E.$Asic.$Ch [set Me]_E.$Asic.$Ch} m]

         }
       }
    }

}

proc map_R3B_spectra {} {
   upvar #0 SpectraHandle f
   global Options
   global NumASICs MaxChans

   set Me $::DataAcquisition::ACQSERVER

   set SpectraDefs [file join /tmp SpectraDefs.dat]

      set z [catch {open $SpectraDefs w} f]
      if {$z != 0} then {
          puts "Cannot open file $SpectraDefs $f"
          return
      }

      map_spectrum "Stat 0 0 $Options(Stat.offset) $Options(Stat.shift)" 

   for {set Asic 0} {$Asic < $NumASICs} {incr Asic} {

      for {set Ch 0} {$Ch < $MaxChans} {incr Ch} {

        if {$Options(R3B_Hist_E_Enable) == 1} {

           set group [expr $Options(R3B_GroupBase)+$Asic]
           set item $Ch
           map_spectrum "E.$Asic.$Ch $group $item $Options(R3B.offset) $Options(R3B.shift)"
       }
     }
   }

   close $f
}

proc doSetUp {} {
    global Options

puts "executing doSetUp"

     Set_Options; make-UIregisters

     DataAcqCommand 1 "Halt"

     unmap_spectra

     delete_spectra

      set Me $::DataAcquisition::ACQSERVER

      CreateSpectrum1D Stat $Options(Stat.channels)
      set z [catch {SpectrumPutTitle Stat [set Me]_Stat} m]

      CreateSpectrum1D Rate $Options(Rate.channels)
      set z [catch {SpectrumPutTitle Rate [set Me]_Rate} m]

    create_R3B_spectra

    map_R3B_spectra 

    map_spectra

    Setup_Electronics

#    ParameterRestore $Options(ASIC.settings)

     DataAcqCommand 3 "Setup"

    WriteNetVar NetVar.DACQ.PollTicks 100
    WriteNetVar NetVar.DACQ.Crdoutenable 1   ;# enable data readout

    puts "completed doSetUp"

}

   proc ParameterRestore {vl} {

      set ::R3B::DBNodeBase $vl
      set ::R3B::DBNodeName $::DataAcquisition::ACQSERVER
      
      puts "Restoring settings from $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"

#    first restore ASIC settings

     set ASICList {1 2 3 5 6 7 9 10 11 13 14 15}

      foreach ASIC $ASICList {

         for {set i 1} {$i <= 5} {incr i} {   ;# avoid REG0
            set vl [DBGetValue ASIC.$ASIC.$i]
            WriteR3BASICRegister [expr ($ASIC << 10) + $i] $vl
         }

         for {set i 10} {$i <= 21} {incr i} {
            set vl [DBGetValue ASIC.$ASIC.$i]
            WriteR3BASICRegister [expr ($ASIC << 10) + $i] $vl
         }

#         for {set i 64} {$i <= 79} {incr i} {
#            set vl [DBGetValue ASIC.$ASIC.$i]
#            WriteR3BASICRegister [expr ($ASIC << 10) + $i] $vl
#         }

#         for {set i 80} {$i <= 95} {incr i} {
#            set vl [DBGetValue ASIC.$ASIC.$i]
#            WriteR3BASICRegister [expr ($ASIC << 10) + $i] $vl
#         }

      }

      puts "Settings restored from $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"


#       ::R3B::WriteRegister HasBeenRestored 1

#        Elog INFORMATION "FEE64 settings restored from $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"

   }

   proc ParameterSave {vl} {

      set ::R3B::DBNodeBase $vl
      set ::R3B::DBNodeName $::DataAcquisition::ACQSERVER
      
      puts "[date +%Y%h%d-%H.%M.%S] Saving settings to $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"

#    first save ASIC settings

     set ASICList {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}

     set settings ""

      foreach ASIC $ASICList {

         for {set i 1} {$i <= 5} {incr i} {   ;# avoid REG0
            set vl [ReadR3BASICRegister [expr ($ASIC << 10) + $i]]
            append settings  "ASIC.$ASIC.$i=[format "0x%02x" [expr $vl & 255]];"
         }

         for {set i 10} {$i <= 21} {incr i} {
            set vl [ReadR3BASICRegister [expr ($ASIC << 10) + $i]]
            append settings  "ASIC.$ASIC.$i=[format "0x%02x" [expr $vl & 255]];"
         }

#         for {set i 64} {$i <= 79} {incr i} {
#            set vl [ReadR3BASICRegister [expr ($ASIC << 10) + $i]]
#            append settings  "ASIC.$ASIC.$i=[format "0x%02x" [expr $vl & 255]];"
#         }

#         for {set i 80} {$i <= 95} {incr i} {
#            set vl [ReadR3BASICRegister [expr ($ASIC << 10) + $i]]
#            append settings  "ASIC.$ASIC.$i=[format "0x%02x" [expr $vl & 255]];"
#         }

      }

      DBSetValue ASIC.settings $settings

      puts "[date +%Y%h%d-%H.%M.%S] Settings saved to $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"

#       ::R3B::WriteRegister HasBeenRestored 1

#        Elog INFORMATION "R3B4K settings saved to $::R3B::DBRootName/$::R3B::DBNodeBase for $::DataAcquisition::ACQSERVER"

   }

proc Reset_Electronics {} {    ;#   used by RESET

    set Me $::DataAcquisition::ACQSERVER

     WriteR3BRegister 0x100 2        ;#  set reset DMA
     WriteR3BRegister 0x100 0        ;#  and clear

     WriteR3BRegister 0x106 0        ;#  and clear rd_ptr

}

proc Setup_Electronics {} {    ;#   used by SETUP

    set Me $::DataAcquisition::ACQSERVER

     WriteR3BRegister 0x100 2        ;#  set reset DMA
     WriteR3BRegister 0x100 0        ;#  and clear

     WriteR3BRegister 0x102 0x0f000000   ;# set DMA MEM Base
     WriteR3BRegister 0x103 2048         ;# set DMA burst length
     WriteR3BRegister 0x105 8192  ;# set DMA MEM Length (units of burst length) (0x01000000/2048) == (16M/2K)
     WriteR3BRegister 0x104 2048         ;# set DMA Buffer Lenghth (units of burst length)   (8192/4)

     WriteR3BRegister 0x106 0        ;#  and clear rd_ptr

}

proc Stop_Electronics {} {

     WriteR3BRegister 0x100 0        ;#    unset enable DMA

}

proc Go_Electronics {} {

    set Me $::DataAcquisition::ACQSERVER

     WriteR3BRegister 0x100 2        ;#  set reset DMA
     WriteR3BRegister 0x100 0        ;#  and clear

     WriteR3BRegister 0x106 0        ;#  and clear rd_ptr

     WriteR3BRegister 0x100 1        ;#  enable  DMA
}

    proc HistogramZero {} {

      set n 0

       set spectra [SpectrumNames]
       foreach item $spectra {
           set name [second $item]
           ZeroSpectrum $name
           incr n
       }

       puts "$n spectra zeroed"
    }


 proc SpecSave1D {spec base range type title counts} {
    variable SpecPath

      set f3 [file join $SpecPath $spec.msf]
      set z1 [catch {open $f3 w} m1]
      set z2 [catch {file attributes $f3 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f3 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f3 - $m2"
      }

      set f $m1

      fconfigure $f -translation binary -encoding binary

  set ctime [clock format [clock seconds] -format {%d-%b-%Y %H:%M:%S}]
  set mtime [clock format [clock seconds] -format {%d-%b-%Y %H:%M:%S}]

      puts -nonewline $f [binary format i1 412900921]   ;#   magic number
      puts -nonewline $f [binary format i1 1]           ;#   version
      puts -nonewline $f [binary format a32 $spec]      ;#   spectrum name
      puts -nonewline $f [binary format i1 1]           ;#   dimension
      puts -nonewline $f [binary format a20 $ctime]     ;#   creation time/date
      puts -nonewline $f [binary format a20 $mtime]     ;#   modification time/date
      puts -nonewline $f [binary format i1 $base]       ;#   base0

      for {set i 2} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#   baseI
      }

      puts -nonewline $f [binary format i1 $range]    ;#   range0

      for {set i 2} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#   rangeI
      }

      puts -nonewline $f [binary format i1 -1]        ;#   spectrum title offset

      for {set i 2} {$i <=32} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  information pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  annotation pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  calibration pointers
      }

      for {set i 1} {$i <=8} {incr i} {
         puts -nonewline $f [binary format i1 -1]     ;#  efficiency pointers
      }

                                                     ;#  data array 1 descriptor 
      puts -nonewline $f [binary format i1 0]        ;#  array layout
      puts -nonewline $f [binary format i1 $type]    ;#  array type  (32 bit unsigned)
      puts -nonewline $f [binary format i1 0]        ;#  reserved 
      puts -nonewline $f [binary format i1 0]        ;#  reserved
      puts -nonewline $f [binary format i1 0]        ;#  spectrum counts offset

                                                     ;#  data array 2 descriptor 
      puts -nonewline $f [binary format i1 0]        ;#  array layout
      puts -nonewline $f [binary format i1 $type]    ;#  array type 
      puts -nonewline $f [binary format i1 0]        ;#  reserved 
      puts -nonewline $f [binary format i1 0]        ;#  reserved
      puts -nonewline $f [binary format i1 -1]       ;#  spectrum counts offset

      puts -nonewline $f [binary format i1 436]      ;#  base of string space
      puts -nonewline $f [binary format i1 20]       ;#  string free space
      puts -nonewline $f [binary format i1 76]       ;#  top of string space

      puts -nonewline $f [binary format i1 512]      ;#  base of counts space
      puts -nonewline $f [binary format i1 $range]   ;#  string free space
      puts -nonewline $f [binary format i1 $range]   ;#  top of string space
 
      puts -nonewline $f [binary format a20 $title]  ;# string space - title

      for {set i 456} {$i < 512} {incr i} {
         puts -nonewline $f [binary format c1 0]     ;#  unused
      }

      puts -nonewline $f [binary format i* $counts]  ;# counts

      close $f
}

    proc HistogramSave {} {
       variable SpecBase
       variable SpecPath

        puts "Saving Histograms"

      set f1 $SpecBase
      set z1 [catch {file mkdir $f1} m1]
      set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f1 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f1 - $m2"
      }

      set f1 [file join $f1 [exec uname -n]]
      set z1 [catch {file mkdir $f1} m1]
      set z2 [catch {file attributes $f1 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f1 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f1 - $m2"
      }

      set f2 [file join $f1 [date +%Y%h%d-%H.%M.%S]]
      set z1 [catch {file mkdir $f2} m1]
      set z2 [catch {file attributes $f2 -permissions rwxrwxrwx} m2]

      if {$z1 != 0} then {
          puts "Cannot create $f2 - $m1"
      }
      if {$z2 != 0} then {
          puts "error accessing $f2 - $m2"
      }

       set SpecPath $f2

       set SpecNames [SpectrumNames]
       set NumSpec 0

      foreach SpecName {Stat} {

         set info [SpectrumDetails $SpecName]
         set channels [lindex $info 2]

         set title [SpectrumGetTitle $SpecName]
         set counts [SpectrumRead1D $SpecName 0 $channels]

         set z [catch {SpecSave1D $SpecName 0 $channels 5 $title $counts} m]
         if {$z!=0} then {
             puts "Error attempting to save spectrum $SpecName: $m"
         } else {
             incr NumSpec
         }

      }

      foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

         set SpecName [second $item]

         set ASIC [second [split $SpecName .]]
         if {$ASIC != 14} {continue}

         set info [SpectrumDetails $SpecName]
         set channels [lindex $info 2]

         set title [SpectrumGetTitle $SpecName]
         set counts [SpectrumRead1D $SpecName 0 $channels]

         set z [catch {SpecSave1D $SpecName 0 $channels 5 $title $counts} m]
         if {$z!=0} then {
             puts "Error attempting to save spectrum $SpecName: $m"
         } else {
             incr NumSpec
         }

      }

        puts "$NumSpec Histograms saved"

    }


    proc TerminateExec {} {

        unmap_spectra
        DataAcqCommand 22 "Terminating DataAcq program" 0 2

    }

    proc LoadExec {} {

       set z [catch {eval exec nice [file join /MIDAS Data_Acq bin linux-ppc_4xx R3BExecV1] &} m]
       if {$z != 0} {puts "RunControl LoadExec returned z=$z and $m"}

    }

    make-UIregisters

    Set_Options




